#include <scl1.h>

/*                       SCL1.LIB's File Functions Demo

    To run this demo inside QuickC environment:

        Copy SCL1.H into your INCLUDE subdirectory.
        Load SCL1 Quick Library.
        Turn Screen Swapping On (QC's Debug Menu).
        Compile it (memory output option and debug option on if you
                    want to step through).


        To Load SCL1 Quick Library:

            qc /l scl1

    File SCL1.QLB must be located in the path specified in the LIB
    environment variable directory or in your default drive/path

    */

char Buffer[512];
char SearchMask[]="\\*.*";

/* During demo FILEDEMO.DAT will be created only if it doesn't exist
   in case the program finds a file with this name it will display an
   error and exit using ErrorBox function */

char OurError[]="FILEDEMO.DAT already exist";

/* structure needed to add our error message to ErrorBox error
   messages list */

struct ErrorMess em[]={
	256,
	OurError};

/* We'll add and change an extension to this filename to using
	AddExtension & ChangeExtension functions */

char Filename[12]="FILE";

main()
{
int i,ErrorCode,Handle;
long FileSize;
unsigned int ReadBytes;
char *p,*s;

/* Clear screen hide and set cursor position */

Cls(7,CLS_ALL);
CursorOff();
SetCurPos(2,20);

/* Set Int 24h (critical error handler) interrupt service routine
   to avoid DOS message "Not ready error reading... in case of
   a critical error (try running this demo with a drive door open)*/

TrapInt24();

/* Write to the screen using file function WriteFile
   Parameters: File Handle (1 = Standard output),
               char pointer to buffer or data to be written,
               number of bytes to write

   Like most SCL1's file related functions WriteFile returns 0 if no
   error, else DOS error code number or -1 in case of a critical
   error (Int 24h) . In this demo function ErrorBox will be used
   to find and display error messages. We'll add one error message
   to ErrorBox default messages. */

InitUserError(em);   /*Call InitUserError with a pointer to our
					   error table */

WriteFile(1,"<< SCL1's file related functions demo >>\r\n\r\n",44);

WriteFile(1,"Original filename: ",19);
WriteFile(1,Filename,4);

/* Let's add an extension. AddExtension parameters: char pointer to filename
	buffer, char pointer to extension. This functions adds an extension only
	if the filename has no extension or does not end with a period. */

AddExtension(Filename,"C");

WriteFile(1,"\r\n\r\nNew filename: ",18);
WriteFile(1,Filename,6);

/* Now let's change the extension using ChangeExtension. Same parameters as
	AddExtension. In case the filename has no extension the extension will be
	added. */

ChangeExtension(Filename,"BAK");

WriteFile(1,"\r\n\r\nWith new extension: ",24);
WriteFile(1,Filename,8);

WriteFile(1,"\r\n\r\nPress any key...\r\n",22);
GetKey();
Cls(7,3,0,6,40);

/* Get Default drive and path
   Prameters: char pointer to buffer that will be filled with
   current drive+path */

if((ErrorCode=GetCurrentDir(Buffer))>0)   /* > 0 means error */
    ErrorBox(ErrorCode);                /* Display error number  */
else if(ErrorCode==-1)                  /* -1 means Critical error */
    {
    Cls(7,CLS_ALL);
    CursorOn();
    exit(ErrorCode);                    /* abort program */
    }

/* Default drive and path is in Buffer, find buffer lenght */

for(i=0,p=Buffer;*p!=0;++i,++p);

/* Add search mask */

s=SearchMask;

if(*(p-1)==92)
    ++s;

for(i=0;i<6;++i,++s,++p)
    *p=*s;

/* Select a file for opening and reading using FileBox
   Parameters: Normal Color, Reverse Color, Filename Buffer */
do
	{
	if((ErrorCode=FileBox(7,0x70,Buffer))>0)
		ErrorBox(ErrorCode);
	else if(ErrorCode==-1)
		{
		Cls(7,CLS_ALL);
		CursorOn();
		exit(ErrorCode);        /* abort program in case of critical error */
		}
	}while(ErrorCode);          /* repeat in case of non-critical error */

Cls(7,5,0,17,79);


/* Open selected file
   Parameters: char pointer to file name, int pointer to File Handle variable
			   char Open Mode (defined in SCL1.H) */

if((ErrorCode=OpenFile(Buffer,&Handle,DOS2_READ))>0)
    ErrorBox(ErrorCode);
else if(ErrorCode==-1)
    {
    Cls(7,CLS_ALL);
    CursorOn();
    exit(ErrorCode);                    /* abort program */
    }

/* Get selected file size
   Parameters: Handle   */

if((FileSize=GetFileSize(Handle))==-1)
    ErrorBox(ErrorCode);

/* Read Data using ReadFile
   Parameters: Handle,char pointer to buffer,int pointer to Bytes variable
   Returns 0 if no error else DOS error code
   Variable Bytes equals number of bytes to read on exit equals number
   of bytes read. */

if (FileSize>512)
    ReadBytes=i=512;
else
    ReadBytes=i=FileSize;

if((ErrorCode=ReadFile(Handle,Buffer,ReadBytes))>0)
	ErrorBox(ErrorCode);        /* Use ErrorBox to display error */
else if(ErrorCode==-1)          /* = -1?, Critical error */
    {
    CloseFile(Handle);
    Cls(7,CLS_ALL);
    CursorOn();
    exit(ErrorCode);                    /* abort program */
    }

/* Close file handle */

if((ErrorCode=CloseFile(Handle))>0)
    ErrorBox(ErrorCode);
else if(ErrorCode==-1)
    {
    Cls(7,CLS_ALL);
    CursorOn();
    exit(ErrorCode);                    /* abort program */
    }

/* Write buffer to the screen */

WriteFile(1,Buffer,ReadBytes);
WriteFile(1,"\r\nPress any key...\r\n",20);
GetKey();

/* Fill Buffer with new data (character A) */

for(i=0,p=Buffer;i<512;++i,++p)
    *p='A';

/* Create a new file named FILEDEMO.DAT with data in buffer. We'll check
   first if a file with that name exist already
   Parameters: char pointer to filename, int pointer to Handle variable
			   file attributte (defined in SCL1.H) */

if(ErrorCode=OpenFile("FILEDEMO.DAT",&Handle,DOS2_RW)>0) /* if file not found */
    {
	if(ErrorCode=CreateFile("FILEDEMO.DAT",&Handle,F_ARCHIVE)>0)       /* create file */
        ErrorBox(ErrorCode);
    else if(ErrorCode==-1)
        {
        Cls(7,CLS_ALL);
        CursorOn();
        exit(ErrorCode);                    /* abort program */
        }
    }
else
    {
    ErrorBox(256);          /* if file exist display our error message */
    Cls(7,CLS_ALL);         /* and exit */
    CursorOn();
    exit(ErrorCode);        /* abort program */
    }

/* Write data in buffer to newly created file using WriteFile */

if((ErrorCode=WriteFile(Handle,Buffer,512))>0)  /* > 0 means error */
    ErrorBox(ErrorCode);                              /* Display error number */
else if(ErrorCode==-1)                         /* -1 means Critical error */
    {
    Cls(7,CLS_ALL);
    CursorOn();
    exit(ErrorCode);                    /* abort program */
    }

CloseFile(Handle);  /* Close file */


/* File operations can also be performed in only one step using File2Buf
   and Buf2Disk. We'll now read the file we have created into our buffer
   using File2Buf and then we'll write it back to disk using Buf2Disk

   File2Buf parameters: char pointer to filename, char pointer to Buffer,
                        int pointer to a variable that equals the maximum
                        number of bytes we can read. On return ReadBytes
                        holds the number of bytes read */

/* Delete Buffer data */

for(i=0,p=Buffer;i<512;++i,++p)
	*p=0;

ReadBytes=512;

/* Read file */

if((ErrorCode=File2Buf("FILEDEMO.DAT",Buffer,&ReadBytes))>0)
    ErrorBox(ErrorCode);
else if(ErrorCode==-1)          /* Critical error */
    {
    Cls(7,CLS_ALL);
    CursorOn();
    exit(ErrorCode);                    /* abort program */
    }

/* Now we'll write the file back to the disk using Buf2Disk
   Parameters:char pointer to filename, char pointer to buffer,
   unsigned int buffer size */

if((ErrorCode=Buf2Disk("FILEDEMO.DAT",Buffer,ReadBytes))>0)
    ErrorBox(ErrorCode);
else if(ErrorCode==-1)          /* Critical error */
    {
    Cls(7,CLS_ALL);
    CursorOn();
    exit(ErrorCode);                    /* abort program */
    }

/* Write buffer to the screen */

WriteFile(1,Buffer,ReadBytes);
WriteFile(1,"\r\n",2);
WriteFile(1,"Press any key...\r\n",18);
GetKey();


/* Finally we'll delete FILEDEMO.DAT using DeleteFile
   Parameters: pointer to filename */

if(ErrorCode=DeleteFile("FILEDEMO.DAT")>0)
    ErrorBox(ErrorCode);
else if(ErrorCode==-1)          /* Critical error */
    {
    Cls(7,CLS_ALL);
    CursorOn();
    exit(ErrorCode);                    /* abort program */
    }
CursorOn();
exit(0);
}
